@@ -1,4 +1,6 @@
 from __future__ import print_function
+from logging import getLogger
+logger = getLogger(__name__)
 import unittest
 import os
 import sys
@@ -7,6 +9,8 @@ import fcntl
 import errno
 import time
 import signal
+import atexit
+import traceback
 from contextlib import contextmanager
 try:
     from cStringIO import StringIO
@@ -144,3 +148,75 @@ class ProcessTestCase(unittest.TestCase):
             print(cb())
             print("******************************")
             raise
+
+_cov = None
+def restart_coverage():
+    logger.critical("(RE)STARTING COVERAGE.")
+    global _cov
+    try:
+        from coverage.control import coverage
+        from coverage.collector import Collector
+    except ImportError:
+        _cov = None
+        return
+    if _cov:
+        _cov.save()
+        _cov.stop()
+    if Collector._collectors:
+        Collector._collectors[-1].stop()
+
+    _cov = coverage(auto_data=True, data_suffix=True, timid=False, include=['src/*'])
+    _cov.start()
+
+    @atexit.register
+    def cleanup():
+        if _cov.collector._collectors:
+            _cov.stop()
+        _cov.save()
+
+def monkeypatch(mod, what):
+    """
+    Patch function named <what> from module <mod> to run the decorated function after <what> completes.
+
+    Eg::
+
+        @monkeypatch(os, 'forkpty')
+        def patched_forkpty(pid_fd):
+            pid, fd = pid_fd
+            if not pid:
+                maybe_enable_coverage()
+            return pid, fd
+    """
+
+    old = getattr(mod, what)
+    def decorator(func):
+        def patch():
+            ret = old()
+            try:
+                func(ret)
+            except:
+                traceback.print_exc()
+            return ret
+        setattr(mod, what, patch)
+    return decorator
+
+def setup_coverage(env_var="WITH_COVERAGE"):
+    """
+    Patch fork and forkpty to restart coverage measurement after fork. Expects to have a environment variable named WITH_covERAGE set to a
+    non-empty value.
+    """
+    if os.environ.get(env_var): # don't even bother if not set
+        restart_coverage()
+
+        @monkeypatch(os, 'fork')
+        def patched_fork(pid):
+            if not pid:
+                restart_coverage()
+            return pid
+
+        @monkeypatch(os, 'forkpty')
+        def patched_forkpty(pid_fd):
+            pid, fd = pid_fd
+            if not pid:
+                restart_coverage()
+            return pid, fd
